@@ -1,10 +1,10 @@
 /*
-   rc-update
-   Manage init scripts and runlevels
-   */
+  rc-update
+  Manage init scripts and runlevels
+*/
 
 /*
- * Copyright 2007-2009 Roy Marples <roy@marples.name>
+ * Copyright (c) 2007-2009 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -55,18 +55,23 @@ add(const char *runlevel, const char *service)
 {
 	int retval = -1;
 
-	if (! rc_service_exists (service))
-		eerror ("%s: service `%s' does not exist", applet, service);
-	else if (rc_service_in_runlevel (service, runlevel)) {
-		ewarn ("%s: %s already installed in runlevel `%s'; skipping",
-		       applet, service, runlevel);
+	if (!rc_service_exists(service)) {
+		if (errno == ENOEXEC)
+			eerror("%s: service `%s' is not executeable",
+			    applet, service);
+		else
+			eerror("%s: service `%s' does not exist",
+			    applet, service);
+	} else if (rc_service_in_runlevel(service, runlevel)) {
+		ewarn("%s: %s already installed in runlevel `%s'; skipping",
+		    applet, service, runlevel);
 		retval = 0;
-	} else if (rc_service_add (runlevel, service)) {
-		einfo ("%s added to runlevel %s", service, runlevel);
+	} else if (rc_service_add(runlevel, service)) {
+		einfo("service %s added to runlevel %s", service, runlevel);
 		retval = 1;
 	} else
-		eerror ("%s: failed to add service `%s' to runlevel `%s': %s",
-			applet, service, runlevel, strerror (errno));
+		eerror("%s: failed to add service `%s' to runlevel `%s': %s",
+		    applet, service, runlevel, strerror (errno));
 
 	return retval;
 }
@@ -77,21 +82,77 @@ delete(const char *runlevel, const char *service)
 	int retval = -1;
 
 	errno = 0;
-	if (rc_service_delete (runlevel, service)) {
-		einfo ("%s removed from runlevel %s", service, runlevel);
+	if (rc_service_delete(runlevel, service)) {
+		einfo("service %s removed from runlevel %s",
+		    service, runlevel);
 		return 1;
 	}
 
 	if (errno == ENOENT)
-		eerror ("%s: service `%s' is not in the runlevel `%s'",
-			applet, service, runlevel);
+		eerror("%s: service `%s' is not in the runlevel `%s'",
+		    applet, service, runlevel);
 	else
-		eerror ("%s: failed to remove service `%s' from runlevel `%s': %s",
-			applet, service, runlevel, strerror (errno));
+		eerror("%s: failed to remove service `%s' from runlevel `%s': %s",
+		    applet, service, runlevel, strerror (errno));
 
 	return retval;
 }
 
+static int
+addstack(const char *runlevel, const char *stack)
+{
+	if (!rc_runlevel_exists(runlevel)) {
+		eerror("%s: runlevel `%s' does not exist", applet, runlevel);
+		return -1;
+	}
+	if (!rc_runlevel_exists(stack)) {
+		eerror("%s: runlevel `%s' does not exist", applet, stack);
+		return -1;
+	}
+	if (strcmp(runlevel, stack) == 0) {
+		eerror("%s: cannot stack `%s' onto itself", applet, stack);
+		return -1;
+	}
+	if (strcmp(runlevel, RC_LEVEL_SYSINIT) == 0 ||
+	    strcmp(stack, RC_LEVEL_SYSINIT) == 0 ||
+	    strcmp(runlevel, RC_LEVEL_BOOT) == 0 ||
+	    strcmp(stack, RC_LEVEL_BOOT) == 0 ||
+	    strcmp(runlevel, RC_LEVEL_SINGLE) == 0 ||
+	    strcmp(stack, RC_LEVEL_SINGLE) == 0 ||
+	    strcmp(runlevel, RC_LEVEL_SHUTDOWN) == 0 ||
+	    strcmp(stack, RC_LEVEL_SHUTDOWN) == 0)
+	{
+		eerror("%s: cannot stack the %s runlevel",
+		    applet, RC_LEVEL_SYSINIT);
+		return -1;
+	}
+	if (!rc_runlevel_stack(runlevel, stack)) {
+		eerror("%s: failed to stack `%s' to `%s': %s",
+		    applet, stack, runlevel, strerror(errno));
+		return -1;
+	}
+	einfo("runlevel %s added to runlevel %s", stack, runlevel);
+	return 1;
+}
+
+static int
+delstack(const char *runlevel, const char *stack)
+{
+	if (rc_runlevel_unstack(runlevel, stack)) {
+		einfo("runlevel %s removed from runlevel %s", stack, runlevel);
+		return 1;
+	}
+
+	if (errno == ENOENT)
+		eerror("%s: runlevel `%s' is not in the runlevel `%s'",
+		    applet, stack, runlevel);
+	else
+		eerror("%s: failed to remove runlevel `%s' from runlevel `%s': %s",
+		    applet, stack, runlevel, strerror (errno));
+
+	return -1;
+}
+
 static void
 show(RC_STRINGLIST *runlevels, bool verbose)
 {
@@ -109,7 +170,7 @@ show(RC_STRINGLIST *runlevels, bool verbose)
 
 		TAILQ_FOREACH(runlevel, runlevels, entries) {
 			if (rc_service_in_runlevel(service->value,
-						   runlevel->value))
+				runlevel->value))
 			{
 				rc_stringlist_add(in, runlevel->value);
 				inone = true;
@@ -124,7 +185,7 @@ show(RC_STRINGLIST *runlevels, bool verbose)
 		if (inone || verbose) {
 			printf(" %20s |", service->value);
 			TAILQ_FOREACH(runlevel, in, entries)
-				printf (" %s", runlevel->value);
+			    printf (" %s", runlevel->value);
 			printf ("\n");
 		}
 		rc_stringlist_free(in);
@@ -134,16 +195,18 @@ show(RC_STRINGLIST *runlevels, bool verbose)
 }
 
 #include "_usage.h"
-#define usagestring "" \
-	"Usage: rc-update [options] add service <runlevel>\n" \
-	"       rc-update [options] del service <runlevel>\n" \
+#define usagestring ""							      \
+	"Usage: rc-update [options] add service <runlevel>\n"		      \
+	"       rc-update [options] del service <runlevel>\n"		      \
 	"       rc-update [options] show"
-#define getoptstring "u" getoptstring_COMMON
+#define getoptstring "su" getoptstring_COMMON
 static const struct option longopts[] = {
+	{ "stack",           0, NULL, 's' },
 	{ "update",          0, NULL, 'u' },
 	longopts_COMMON
 };
 static const char * const longopts_help[] = {
+	"Stack a runlevel instead of a service",
 	"Force an update of the dependency tree",
 	longopts_help_COMMON
 };
@@ -161,7 +224,7 @@ rc_update(int argc, char **argv)
 	char *service = NULL;
 	char *p;
 	int action = 0;
-	bool verbose = false;
+	bool verbose = false, stack = false;
 	int opt;
 	int retval = EXIT_FAILURE;
 	int num_updated = 0;
@@ -169,12 +232,15 @@ rc_update(int argc, char **argv)
 	int ret;
 
 	while ((opt = getopt_long(argc, argv, getoptstring,
-				  longopts, (int *)0)) != -1)
+		    longopts, (int *)0)) != -1)
 		switch (opt) {
+		case 's':
+		    stack = true;
+		break;
 		case 'u':
 			_rc_deptree_load(-1, &ret);
 			return ret;
-		case_RC_COMMON_GETOPT
+			case_RC_COMMON_GETOPT;
 		}
 
 	verbose = rc_yesno(getenv ("EINFO_VERBOSE"));
@@ -189,14 +255,15 @@ rc_update(int argc, char **argv)
 		if (strcmp(argv[optind], "add") == 0)
 			action = DOADD;
 		else if (strcmp(argv[optind], "delete") == 0 ||
-			 strcmp(argv[optind], "del") == 0)
+		    strcmp(argv[optind], "del") == 0)
 			action = DODELETE;
 		else if (strcmp(argv[optind], "show") == 0)
 			action = DOSHOW;
 		if (action)
 			optind++;
 		else
-			eerrorx("%s: invalid command `%s'", applet, argv[optind]);
+			eerrorx("%s: invalid command `%s'",
+			    applet, argv[optind]);
 	}
 	if (!action)
 		action = DOSHOW;
@@ -204,7 +271,7 @@ rc_update(int argc, char **argv)
 	runlevels = rc_stringlist_new();
 
 	if (optind >= argc) {
-		if (! action & DOSHOW)
+		if (!(action & DOSHOW))
 			eerrorx("%s: no service specified", applet);
 	} else {
 		service = argv[optind];
@@ -216,7 +283,7 @@ rc_update(int argc, char **argv)
 			else {
 				rc_stringlist_free(runlevels);
 				eerrorx ("%s: `%s' is not a valid runlevel",
-					 applet, argv[optind]);
+				    applet, argv[optind]);
 			}
 	}
 
@@ -235,9 +302,9 @@ rc_update(int argc, char **argv)
 			eerror ("%s: no service specified", applet);
 		else {
 			if (action & DOADD) {
-				actfunc = add;
+				actfunc = stack ? addstack : add;
 			} else if (action & DODELETE) {
-				actfunc = delete;
+				actfunc = stack ? delstack : delete;
 			} else {
 				rc_stringlist_free(runlevels);
 				eerrorx("%s: invalid action", applet);
@@ -251,13 +318,13 @@ rc_update(int argc, char **argv)
 
 			if (!TAILQ_FIRST(runlevels)) {
 				free(runlevels);
-				eerrorx ("%s: no runlevels found", applet);
+				eerrorx("%s: no runlevels found", applet);
 			}
 
 			TAILQ_FOREACH(runlevel, runlevels, entries) {
 				if (!rc_runlevel_exists(runlevel->value)) {
 					eerror ("%s: runlevel `%s' does not exist",
-						applet, runlevel->value);
+					    applet, runlevel->value);
 					continue;
 				}
 
@@ -270,8 +337,8 @@ rc_update(int argc, char **argv)
 			if (retval == EXIT_SUCCESS &&
 			    num_updated == 0 && action & DODELETE)
 				ewarnx("%s: service `%s' not found in any"
-				       " of the specified runlevels",
-				       applet, service);
+				    " of the specified runlevels",
+				    applet, service);
 		}
 	}
 
