@@ -9,7 +9,7 @@
  */
 
 /*
- * Copyright 2007-2009 Roy Marples <roy@marples.name>
+ * Copyright (c) 2007-2009 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -43,16 +43,13 @@ const char rc_copyright[] = "Copyright (c) 2007-2008 Roy Marples";
 #include <sys/utsname.h>
 #include <sys/wait.h>
 
-#ifdef __linux__
-# include <asm/setup.h> /* for COMMAND_LINE_SIZE */
-#endif
-
 #include <errno.h>
 #include <dirent.h>
 #include <ctype.h>
 #include <getopt.h>
 #include <libgen.h>
 #include <limits.h>
+#include <pwd.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -71,8 +68,8 @@ const char rc_copyright[] = "Copyright (c) 2007-2008 Roy Marples";
 
 #include "version.h"
 
-#define INITSH                  RC_LIBDIR "/sh/init.sh"
-#define INITEARLYSH             RC_LIBDIR "/sh/init-early.sh"
+#define INITSH                  RC_LIBEXECDIR "/sh/init.sh"
+#define INITEARLYSH             RC_LIBEXECDIR "/sh/init-early.sh"
 
 #define SHUTDOWN                "/sbin/shutdown"
 #define SULOGIN                 "/sbin/sulogin"
@@ -108,17 +105,17 @@ clean_failed(void)
 		while ((d = readdir(dp))) {
 			if (d->d_name[0] == '.' &&
 			    (d->d_name[1] == '\0' ||
-			     (d->d_name[1] == '.' && d->d_name[2] == '\0')))
+				(d->d_name[1] == '.' && d->d_name[2] == '\0')))
 				continue;
 
 			l = strlen(RC_SVCDIR "/failed/") +
-				strlen(d->d_name) + 1;
+			    strlen(d->d_name) + 1;
 			path = xmalloc(sizeof(char) * l);
 			snprintf(path, l, RC_SVCDIR "/failed/%s", d->d_name);
 			if (path) {
 				if (unlink(path))
 					eerror("%s: unlink `%s': %s",
-					       applet, path, strerror(errno));
+					    applet, path, strerror(errno));
 				free(path);
 			}
 		}
@@ -176,10 +173,8 @@ static char *
 proc_getent(const char *ent)
 {
 	FILE *fp;
-	char proc[COMMAND_LINE_SIZE];
-	char *p;
-	char *value = NULL;
-	int i;
+	char *proc, *p, *value = NULL;
+	size_t i;
 
 	if (!exists("/proc/cmdline"))
 		return NULL;
@@ -189,8 +184,10 @@ proc_getent(const char *ent)
 		return NULL;
 	}
 
-	memset(proc, 0, sizeof(proc));
-	fgets(proc, sizeof(proc), fp);
+	proc = NULL;
+	i = 0;
+	if (rc_getline(&proc, &i, fp) == -1 || proc == NULL)
+		eerror("rc_getline: %s", strerror(errno));
 	if (*proc && (p = strstr(proc, ent))) {
 		i = p - proc;
 		if (i == '\0' || proc[i - 1] == ' ') {
@@ -202,6 +199,7 @@ proc_getent(const char *ent)
 	} else
 		errno = ENOENT;
 	fclose(fp);
+	free(proc);
 
 	return value;
 }
@@ -217,8 +215,8 @@ read_key(bool block)
 	if (!isatty(fd))
 		return false;
 
-	/* Now save our terminal settings. We need to restore them at exit as we
-	 * will be changing it for non-blocking reads for Interactive */
+	/* Now save our terminal settings. We need to restore them at exit as
+	   we will be changing it for non-blocking reads for Interactive */
 	if (!termios_orig) {
 		termios_orig = xmalloc(sizeof(*termios_orig));
 		tcgetattr(fd, termios_orig);
@@ -233,7 +231,8 @@ read_key(bool block)
 		termios.c_cc[VTIME] = 0;
 	}
 	tcsetattr(fd, TCSANOW, &termios);
-	read(fd, &c, 1);
+	if (read(fd, &c, 1) == -1)
+		eerror("read: %s", strerror(errno));
 	tcsetattr(fd, TCSANOW, termios_orig);
 	return c;
 }
@@ -299,9 +298,9 @@ run_program(const char *prog)
 		if (termios_orig)
 			tcsetattr(STDIN_FILENO, TCSANOW, termios_orig);
 
-		execl(prog, prog, (char *) NULL);
+		execl(prog, prog, (char *)NULL);
 		eerror("%s: unable to exec `%s': %s", applet, prog,
-		       strerror(errno));
+		    strerror(errno));
 		_exit(EXIT_FAILURE);
 	}
 
@@ -312,29 +311,37 @@ run_program(const char *prog)
 }
 
 static void
-sulogin(bool cont)
+open_shell(void)
 {
+	const char *shell;
+	struct passwd *pw;
+	
 #ifdef __linux__
 	const char *sys = rc_sys();
 
-	/* VSERVER and OPENVZ systems cannot do a sulogin */
+	/* VSERVER and OPENVZ systems cannot really drop to shells */
 	if (sys &&
 	    (strcmp(sys, "VSERVER") == 0 || strcmp(sys, "OPENVZ") == 0))
 	{
 		execl("/sbin/halt", "/sbin/halt", "-f", (char *) NULL);
 		eerrorx("%s: unable to exec `/sbin/halt': %s",
-			applet, strerror(errno));
+		    applet, strerror(errno));
 	}
 #endif
-	if (!cont) {
-		rc_logger_close();
-		exit(EXIT_SUCCESS);
+
+	shell = rc_conf_value("rc_shell");
+	/* No shell set, so obey env, then passwd, then default to /bin/sh */
+	if (shell == NULL) {
+		shell = getenv("SHELL");
+		if (shell == NULL) {
+			pw = getpwuid(getuid());
+			if (pw)
+				shell = pw->pw_shell;
+			if (shell == NULL)
+				shell = "/bin/sh";
+		}
 	}
-#ifdef __linux__
-	run_program(SULOGIN);
-#else
-	run_program("/bin/sh");
-#endif
+	run_program(shell);
 }
 
 _dead static void
@@ -343,7 +350,7 @@ single_user(void)
 	rc_logger_close();
 	execl(SHUTDOWN, SHUTDOWN, "now", (char *) NULL);
 	eerrorx("%s: unable to exec `" SHUTDOWN "': %s",
-		 applet, strerror(errno));
+	    applet, strerror(errno));
 }
 
 static bool
@@ -359,7 +366,7 @@ set_krunlevel(const char *level)
 		if (exists(RC_KRUNLEVEL) &&
 		    unlink(RC_KRUNLEVEL) != 0)
 			eerror("unlink `%s': %s", RC_KRUNLEVEL,
-			       strerror(errno));
+			    strerror(errno));
 		return false;
 	}
 
@@ -409,11 +416,11 @@ remove_pid(pid_t pid)
 	RC_PID *p;
 
 	LIST_FOREACH(p, &service_pids, entries)
-		if (p->pid == pid) {
-			LIST_REMOVE(p, entries);
-			free(p);
-			return;
-		}
+	    if (p->pid == pid) {
+		    LIST_REMOVE(p, entries);
+		    free(p);
+		    return;
+	    }
 }
 
 static void
@@ -484,7 +491,7 @@ handle_signal(int sig)
 
 		/* Kill any running services we have started */
 		LIST_FOREACH(pi, &service_pids, entries)
-			kill(pi->pid, SIGTERM);
+		    kill(pi->pid, SIGTERM);
 
 		/* Notify plugins we are aborting */
 		rc_plugin_run(RC_HOOK_ABORT, NULL);
@@ -514,15 +521,15 @@ do_sysinit()
 
 	uname(&uts);
 	printf("\n   %sOpenRC %s" VERSION "%s is starting up %s",
-	       ecolor(ECOLOR_GOOD), ecolor(ECOLOR_HILITE),
-	       ecolor(ECOLOR_NORMAL), ecolor(ECOLOR_BRACKET));
+	    ecolor(ECOLOR_GOOD), ecolor(ECOLOR_HILITE),
+	    ecolor(ECOLOR_NORMAL), ecolor(ECOLOR_BRACKET));
 #ifdef BRANDING
 	printf(BRANDING " (%s)", uts.machine);
 #else
 	printf("%s %s (%s)",
-	       uts.sysname,
-	       uts.release,
-	       uts.machine);
+	    uts.sysname,
+	    uts.release,
+	    uts.machine);
 #endif
 
 	if ((sys = rc_sys()))
@@ -533,7 +540,7 @@ do_sysinit()
 	if (!rc_yesno(getenv ("EINFO_QUIET")) &&
 	    rc_conf_yesno("rc_interactive"))
 		printf("Press %sI%s to enter interactive boot mode\n\n",
-		       ecolor(ECOLOR_GOOD), ecolor(ECOLOR_NORMAL));
+		    ecolor(ECOLOR_GOOD), ecolor(ECOLOR_NORMAL));
 
 	setenv("RC_RUNLEVEL", RC_LEVEL_SYSINIT, 1);
 	run_program(INITSH);
@@ -564,19 +571,22 @@ runlevel_config(const char *service, const char *level)
 }
 
 static void
-do_stop_services(const char *newlevel, bool parallel)
+do_stop_services(const char *newlevel, bool parallel, bool going_down)
 {
 	pid_t pid;
 	RC_STRING *service, *svc1, *svc2;
-	RC_STRINGLIST *deporder, *tmplist;
+	RC_STRINGLIST *deporder, *tmplist, *kwords;
 	RC_SERVICE state;
 	RC_STRINGLIST *nostop;
+	bool crashed, nstop;
 
 	if (!types_n) {
 		types_n = rc_stringlist_new();
 		rc_stringlist_add(types_n, "needsme");
 	}
 
+	crashed = rc_conf_yesno("rc_crashed_stop");
+	
 	nostop = rc_stringlist_split(rc_conf_value("rc_nostop"), " ");
 	TAILQ_FOREACH_REVERSE(service, stop_services, rc_stringlist, entries)
 	{
@@ -589,6 +599,26 @@ do_stop_services(const char *newlevel, bool parallel)
 			rc_service_mark(service->value, RC_SERVICE_FAILED);
 			continue;
 		}
+		kwords = rc_deptree_depend(deptree, service->value, "keyword");
+		if (rc_stringlist_find(kwords, "-stop") ||
+		    rc_stringlist_find(kwords, "nostop") ||
+		    (going_down &&
+			(rc_stringlist_find(kwords, "-shutdown") ||
+			    rc_stringlist_find(kwords, "noshutdown"))))
+			nstop = true;
+		else
+			nstop = false;
+		rc_stringlist_free(kwords);
+		if (nstop) {
+			rc_service_mark(service->value, RC_SERVICE_FAILED);
+			continue;
+		}
+
+		/* If the service has crashed, skip futher checks and just stop
+		   it */
+		if (crashed &&
+		    rc_service_daemons_crashed(service->value))
+			goto stop;
 
 		/* If we're in the start list then don't bother stopping us */
 		svc1 = rc_stringlist_find(start_services, service->value);
@@ -611,13 +641,14 @@ do_stop_services(const char *newlevel, bool parallel)
 		if (!svc1) {
 			tmplist = rc_stringlist_new();
 			rc_stringlist_add(tmplist, service->value);
-			deporder = rc_deptree_depends(deptree, types_n, tmplist,
-						      newlevel ? newlevel : runlevel,
-						      RC_DEP_STRICT | RC_DEP_TRACE);
+			deporder = rc_deptree_depends(deptree, types_n,
+			    tmplist, newlevel ? newlevel : runlevel,
+			    RC_DEP_STRICT | RC_DEP_TRACE);
 			rc_stringlist_free(tmplist);
 			svc2 = NULL;
 			TAILQ_FOREACH(svc1, deporder, entries) {
-				svc2 = rc_stringlist_find(start_services, svc1->value);
+				svc2 = rc_stringlist_find(start_services,
+				    svc1->value);
 				if (svc2)
 					break;
 			}
@@ -627,6 +658,7 @@ do_stop_services(const char *newlevel, bool parallel)
 				continue;
 		}
 
+stop:
 		/* After all that we can finally stop the blighter! */
 		pid = service_stop(service->value);
 		if (pid > 0) {
@@ -648,34 +680,46 @@ do_start_services(bool parallel)
 	pid_t pid;
 	bool interactive = false;
 	RC_SERVICE state;
+	bool crashed = false;
 
 	if (!rc_yesno(getenv("EINFO_QUIET")))
 		interactive = exists(INTERACTIVE);
+	errno = 0;
+	crashed = rc_conf_yesno("rc_crashed_start");
+	if (errno == ENOENT)
+		crashed = true;
 
 	TAILQ_FOREACH(service, start_services, entries) {
 		state = rc_service_state(service->value);
-		if (!(state & RC_SERVICE_STOPPED) || state & RC_SERVICE_FAILED)
+		if (state & RC_SERVICE_FAILED)
 			continue;
-
+		if (!(state & RC_SERVICE_STOPPED)) {
+			if (crashed &&
+			    rc_service_daemons_crashed(service->value))
+				rc_service_mark(service->value,
+				    RC_SERVICE_STOPPED);
+			else
+			    continue;
+		}
 		if (!interactive)
 			interactive = want_interactive();
 
 		if (interactive) {
-interactive_retry:
+	interactive_retry:
 			printf("\n");
 			einfo("About to start the service %s",
-			      service->value);
+			    service->value);
 			eindent();
 			einfo("1) Start the service\t\t2) Skip the service");
 			einfo("3) Continue boot process\t\t4) Exit to shell");
 			eoutdent();
-interactive_option:
+	interactive_option:
 			switch (read_key(true)) {
-				case '1': break;
-				case '2': continue;
-				case '3': interactive = false; break;
-				case '4': sulogin(true); goto interactive_retry;
-				default: goto interactive_option;
+			case '1': break;
+			case '2': continue;
+			case '3': interactive = false; break;
+			case '4': open_shell(); goto interactive_retry;
+			default: goto interactive_option;
 			}
 		}
 
@@ -693,7 +737,7 @@ interactive_option:
 	/* Store our interactive status for boot */
 	if (interactive &&
 	    (strcmp(runlevel, RC_LEVEL_SYSINIT) == 0 ||
-	     strcmp(runlevel, getenv("RC_BOOTLEVEL")) == 0))
+		strcmp(runlevel, getenv("RC_BOOTLEVEL")) == 0))
 		mark_interactive();
 	else {
 		if (exists(INTERACTIVE))
@@ -711,17 +755,17 @@ handle_bad_signal(int sig)
 	pid_t crashed_pid = getpid();
 
 	switch (fork()) {
-		case -1:
-			_exit(sig);
-			/* NOTREACHED */
-		case 0:
-			sprintf(pid, "%i", crashed_pid);
-			printf("\nAuto launching gdb!\n\n");
-			_exit(execlp("gdb", "gdb", "--quiet", "--pid", pid,
-				     "-ex", "bt full", NULL));
-			/* NOTREACHED */
-		default:
-			wait(&status);
+	case -1:
+		_exit(sig);
+		/* NOTREACHED */
+	case 0:
+		sprintf(pid, "%i", crashed_pid);
+		printf("\nAuto launching gdb!\n\n");
+		_exit(execlp("gdb", "gdb", "--quiet", "--pid", pid,
+			"-ex", "bt full", NULL));
+		/* NOTREACHED */
+	default:
+		wait(&status);
 	}
 	_exit(1);
 	/* NOTREACHED */
@@ -737,7 +781,8 @@ static const struct option longopts[] = {
 	longopts_COMMON
 };
 static const char * const longopts_help[] = {
-	"override the next runlevel to change into\nwhen leaving single user or boot runlevels",
+	"override the next runlevel to change into\n"
+	"when leaving single user or boot runlevels",
 	"runs the service specified with the rest\nof the arguments",
 	"output the RC system type, if any",
 	longopts_help_COMMON
@@ -783,9 +828,9 @@ main(int argc, char **argv)
 			printf(" [%s]", bootlevel);
 		printf(") " VERSION
 #ifdef BRANDING
-		       " (" BRANDING ")"
+		    " (" BRANDING ")"
 #endif
-		       "\n");
+		    "\n");
 		exit(EXIT_SUCCESS);
 	}
 
@@ -796,7 +841,8 @@ main(int argc, char **argv)
 	argv++;
 
 	/* Change dir to / to ensure all scripts don't use stuff in pwd */
-	chdir("/");
+	if (chdir("/") == -1)
+		eerror("chdir: %s", strerror(errno));
 
 	/* Ensure our environment is pure
 	 * Also, add our configuration to it */
@@ -806,7 +852,7 @@ main(int argc, char **argv)
 	argc++;
 	argv--;
 	while ((opt = getopt_long(argc, argv, getoptstring,
-				  longopts, (int *) 0)) != -1)
+		    longopts, (int *) 0)) != -1)
 	{
 		switch (opt) {
 		case 'o':
@@ -825,7 +871,7 @@ main(int argc, char **argv)
 			newlevel = rc_service_resolve(optarg);
 			if (!newlevel)
 				eerrorx("%s: service `%s' does not exist",
-					applet, optarg);
+				    applet, optarg);
 			argv += optind - 1;
 			*argv = newlevel;
 			execv(*argv, argv);
@@ -837,8 +883,8 @@ main(int argc, char **argv)
 				printf("%s\n", bootlevel);
 			exit(EXIT_SUCCESS);
 			/* NOTREACHED */
-		case_RC_COMMON_GETOPT
-		}
+			case_RC_COMMON_GETOPT
+			    }
 	}
 
 	newlevel = argv[optind++];
@@ -886,7 +932,7 @@ main(int argc, char **argv)
 
 	if (newlevel &&
 	    (strcmp(newlevel, RC_LEVEL_SHUTDOWN) == 0 ||
-	     strcmp(newlevel, RC_LEVEL_SINGLE) == 0))
+		strcmp(newlevel, RC_LEVEL_SINGLE) == 0))
 	{
 		going_down = true;
 		if (!exists(RC_KRUNLEVEL))
@@ -895,10 +941,10 @@ main(int argc, char **argv)
 		setenv("RC_RUNLEVEL", newlevel, 1);
 		setenv("RC_GOINGDOWN", "YES", 1);
 	} else {
-		/* We should not use krunevel in sysinit or the boot runlevel */
+		/* We should not use krunevel in sysinit or boot runlevels */
 		if (!newlevel ||
 		    (strcmp(newlevel, RC_LEVEL_SYSINIT) != 0 &&
-		     strcmp(newlevel, getenv("RC_BOOTLEVEL")) != 0))
+			strcmp(newlevel, getenv("RC_BOOTLEVEL")) != 0))
 		{
 			if (get_krunlevel(krunlevel, sizeof(krunlevel))) {
 				newlevel = krunlevel;
@@ -960,7 +1006,7 @@ main(int argc, char **argv)
 		if (errno == EACCES)
 			eerrorx("%s: superuser access required", applet);
 		eerrorx("%s: failed to create stopping dir `%s': %s",
-			applet, RC_STOPPING, strerror(errno));
+		    applet, RC_STOPPING, strerror(errno));
 	}
 
 	/* Build a list of all services to stop and then work out the
@@ -982,15 +1028,15 @@ main(int argc, char **argv)
 
 	if (stop_services) {
 		tmplist = rc_deptree_depends(deptree, types_nua, stop_services,
-				runlevel, depoptions | RC_DEP_STOP);
+		    runlevel, depoptions | RC_DEP_STOP);
 		rc_stringlist_free(stop_services);
 		stop_services = tmplist;
 	}
 
 	/* Load our list of start services */
 	hotplugged_services = rc_services_in_state(RC_SERVICE_HOTPLUGGED);
-	start_services = rc_services_in_runlevel(newlevel ?
-						 newlevel : runlevel);
+	start_services = rc_services_in_runlevel_stacked(newlevel ?
+	    newlevel : runlevel);
 	if (strcmp(newlevel ? newlevel : runlevel, RC_LEVEL_SHUTDOWN) != 0 &&
 	    strcmp(newlevel ? newlevel : runlevel, RC_LEVEL_SYSINIT) != 0)
 	{
@@ -998,10 +1044,10 @@ main(int argc, char **argv)
 		TAILQ_CONCAT(start_services, tmplist, entries);
 		free(tmplist);
 		if (strcmp(newlevel ? newlevel : runlevel,
-			   RC_LEVEL_SINGLE) != 0)
+			RC_LEVEL_SINGLE) != 0)
 		{
 			if (strcmp(newlevel ? newlevel : runlevel,
-				   bootlevel) != 0)
+				bootlevel) != 0)
 			{
 				tmplist = rc_services_in_runlevel(bootlevel);
 				TAILQ_CONCAT(start_services, tmplist, entries);
@@ -1009,9 +1055,9 @@ main(int argc, char **argv)
 			}
 			if (hotplugged_services) {
 				TAILQ_FOREACH(service, hotplugged_services,
-					      entries)
-					rc_stringlist_addu(start_services,
-							   service->value);
+				    entries)
+				    rc_stringlist_addu(start_services,
+					service->value);
 			}
 		}
 	}
@@ -1020,14 +1066,14 @@ main(int argc, char **argv)
 
 	/* Now stop the services that shouldn't be running */
 	if (stop_services)
-		do_stop_services(newlevel, parallel);
+		do_stop_services(newlevel, parallel, going_down);
 
 	/* Wait for our services to finish */
 	wait_for_services();
 
 	/* Notify the plugins we have finished */
 	rc_plugin_run(RC_HOOK_RUNLEVEL_STOP_OUT,
-		      going_down ? newlevel : runlevel);
+	    going_down ? newlevel : runlevel);
 	hook_out = 0;
 
 	rmdir(RC_STOPPING);
@@ -1054,14 +1100,14 @@ main(int argc, char **argv)
 	/* Re-add our hotplugged services if they stopped */
 	if (hotplugged_services)
 		TAILQ_FOREACH(service, hotplugged_services, entries)
-			rc_service_mark(service->value, RC_SERVICE_HOTPLUGGED);
+		    rc_service_mark(service->value, RC_SERVICE_HOTPLUGGED);
 
 	/* Order the services to start */
 	if (start_services) {
 		rc_stringlist_sort(&start_services);
 		deporder = rc_deptree_depends(deptree, types_nua,
-					      start_services, runlevel,
-					      depoptions | RC_DEP_START);
+		    start_services, runlevel,
+		    depoptions | RC_DEP_START);
 		rc_stringlist_free(start_services);
 		start_services = deporder;
 	}
