@@ -1,14 +1,14 @@
 /*
-   start-stop-daemon
-   Starts, stops, tests and signals daemons
+  start-stop-daemon
+  Starts, stops, tests and signals daemons
 
-   This is essentially a ground up re-write of Debians
-   start-stop-daemon for cleaner code and to integrate into our RC
-   system so we can monitor daemons a little.
-   */
+  This is essentially a ground up re-write of Debians
+  start-stop-daemon for cleaner code and to integrate into our RC
+  system so we can monitor daemons a little.
+*/
 
 /*
- * Copyright 2007-2009 Roy Marples <roy@marples.name>
+ * Copyright (c) 2007-2009 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -75,9 +75,9 @@ static struct pam_conv conv = { NULL, NULL};
 
 /* Some libc implementations don't define this */
 #ifndef LIST_FOREACH_SAFE
-#define	LIST_FOREACH_SAFE(var, head, field, tvar)			\
-	for ((var) = LIST_FIRST((head));				\
-	     (var) && ((tvar) = LIST_NEXT((var), field), 1);		\
+#define	LIST_FOREACH_SAFE(var, head, field, tvar)			      \
+	for ((var) = LIST_FIRST((head));				      \
+	     (var) && ((tvar) = LIST_NEXT((var), field), 1);		      \
 	     (var) = (tvar))
 #endif
 
@@ -85,12 +85,12 @@ static struct pam_conv conv = { NULL, NULL};
 typedef struct scheduleitem
 {
 	enum
-	{
-		SC_TIMEOUT,
-		SC_SIGNAL,
-		SC_GOTO,
-		SC_FOREVER
-	} type;
+		{
+			SC_TIMEOUT,
+			SC_SIGNAL,
+			SC_GOTO,
+			SC_FOREVER
+		} type;
 	int value;
 	struct scheduleitem *gotoitem;
 	TAILQ_ENTRY(scheduleitem) entries;
@@ -103,7 +103,8 @@ static char *changeuser, *ch_root, *ch_dir;
 
 extern char **environ;
 
-static void free_schedulelist(void)
+static void
+free_schedulelist(void)
 {
 	SCHEDULEITEM *s1 = TAILQ_FIRST(&schedule);
 	SCHEDULEITEM *s2;
@@ -117,7 +118,8 @@ static void free_schedulelist(void)
 }
 
 #ifdef DEBUG_MEMORY
-static void cleanup(void)
+static void
+cleanup(void)
 {
 	free(changeuser);
 	free(nav);
@@ -125,7 +127,8 @@ static void cleanup(void)
 }
 #endif
 
-static int parse_signal(const char *sig)
+static int
+parse_signal(const char *sig)
 {
 	typedef struct signalpair
 	{
@@ -152,7 +155,8 @@ static int parse_signal(const char *sig)
 		{ "STOP",	SIGSTOP	},
 		{ "TSTP",	SIGTSTP	},
 		{ "TTIN",	SIGTTIN	},
-		{ "TTOU",	SIGTTOU	}
+		{ "TTOU",	SIGTTOU	},
+		{ "NULL",	0 },
 	};
 
 	unsigned int i = 0;
@@ -162,7 +166,7 @@ static int parse_signal(const char *sig)
 		return -1;
 
 	if (sscanf(sig, "%u", &i) == 1) {
-		if (i > 0 && i < sizeof(signallist) / sizeof(signallist[0]))
+		if (i < NSIG)
 			return i;
 		eerrorx("%s: `%s' is not a valid signal", applet, sig);
 	}
@@ -181,7 +185,8 @@ static int parse_signal(const char *sig)
 	/* NOTREACHED */
 }
 
-static SCHEDULEITEM *parse_schedule_item(const char *string)
+static SCHEDULEITEM *
+parse_schedule_item(const char *string)
 {
 	const char *after_hyph;
 	int sig;
@@ -195,21 +200,21 @@ static SCHEDULEITEM *parse_schedule_item(const char *string)
 		item->type = SC_TIMEOUT;
 		errno = 0;
 		if (sscanf(string, "%d", &item->value) != 1)
-			eerrorx("%s: invalid timeout value in schedule `%s'", applet,
-				string);
+			eerrorx("%s: invalid timeout value in schedule `%s'",
+			    applet, string);
 	} else if ((after_hyph = string + (string[0] == '-')) &&
-		   ((sig = parse_signal(after_hyph)) != -1))
+	    ((sig = parse_signal(after_hyph)) != -1))
 	{
 		item->type = SC_SIGNAL;
 		item->value = (int)sig;
-	}
-	else
+	} else
 		eerrorx("%s: invalid schedule item `%s'", applet, string);
 
 	return item;
 }
 
-static void parse_schedule(const char *string, int timeout)
+static void
+parse_schedule(const char *string, int timeout)
 {
 	char buffer[20];
 	const char *slash;
@@ -238,7 +243,8 @@ static void parse_schedule(const char *string, int timeout)
 		TAILQ_INSERT_TAIL(&schedule, item, entries);
 		if (string) {
 			if (sscanf(string, "%d", &item->value) != 1)
-				eerrorx("%s: invalid timeout value in schedule", applet);
+				eerrorx("%s: invalid timeout in schedule",
+				    applet);
 		} else
 			item->value = 5;
 
@@ -251,8 +257,9 @@ static void parse_schedule(const char *string, int timeout)
 		else
 			len = strlen(string);
 
-		if (len >= (ptrdiff_t) sizeof(buffer))
-			eerrorx("%s: invalid schedule item, far too long", applet);
+		if (len >= (ptrdiff_t)sizeof(buffer))
+			eerrorx("%s: invalid schedule item, far too long",
+			    applet);
 
 		memcpy(buffer, string, len);
 		buffer[len] = 0;
@@ -263,7 +270,7 @@ static void parse_schedule(const char *string, int timeout)
 		if (item->type == SC_FOREVER) {
 			if (repeatat)
 				eerrorx("%s: invalid schedule, `forever' "
-					"appears more than once", applet);
+				    "appears more than once", applet);
 
 			repeatat = item;
 			continue;
@@ -281,7 +288,8 @@ static void parse_schedule(const char *string, int timeout)
 	return;
 }
 
-static pid_t get_pid(const char *pidfile, bool quiet)
+static pid_t
+get_pid(const char *pidfile, bool quiet)
 {
 	FILE *fp;
 	pid_t pid;
@@ -291,7 +299,8 @@ static pid_t get_pid(const char *pidfile, bool quiet)
 
 	if ((fp = fopen(pidfile, "r")) == NULL) {
 		if (!quiet)
-			eerror("%s: fopen `%s': %s", applet, pidfile, strerror(errno));
+			eerror("%s: fopen `%s': %s",
+			    applet, pidfile, strerror(errno));
 		return -1;
 	}
 
@@ -308,9 +317,10 @@ static pid_t get_pid(const char *pidfile, bool quiet)
 }
 
 /* return number of processed killed, -1 on error */
-static int do_stop(const char *exec, const char *const *argv,
-		   pid_t pid, uid_t uid,int sig,
-		   bool quiet, bool verbose, bool test)
+static int
+do_stop(const char *exec, const char *const *argv,
+    pid_t pid, uid_t uid,int sig,
+    bool quiet, bool verbose, bool test)
 {
 	RC_PIDLIST *pids;
 	RC_PID *pi;
@@ -330,19 +340,19 @@ static int do_stop(const char *exec, const char *const *argv,
 		if (test) {
 			if (!quiet)
 				einfo("Would send signal %d to PID %d",
-				      sig, pi->pid);
+				    sig, pi->pid);
 			nkilled++;
 		} else {
 			if (verbose)
 				ebegin("Sending signal %d to PID %d",
-				sig, pi->pid);
+				    sig, pi->pid);
 			errno = 0;
 			killed = (kill(pi->pid, sig) == 0 ||
-					errno == ESRCH ? true : false);
+			    errno == ESRCH ? true : false);
 			if (verbose)
 				eend(killed ? 0 : 1,
-				     "%s: failed to send signal %d to PID %d: %s",
-				     applet, sig, pi->pid, strerror(errno));
+				    "%s: failed to send signal %d to PID %d: %s",
+				    applet, sig, pi->pid, strerror(errno));
 			if (!killed) {
 				nkilled = -1;
 			} else {
@@ -357,18 +367,20 @@ static int do_stop(const char *exec, const char *const *argv,
 	return nkilled;
 }
 
-static int run_stop_schedule(const char *exec, const char *const *argv,
-			     const char *pidfile, uid_t uid,
-			     bool quiet, bool verbose, bool test)
+static int
+run_stop_schedule(const char *exec, const char *const *argv,
+    const char *pidfile, uid_t uid,
+    bool quiet, bool verbose, bool test, bool progress)
 {
 	SCHEDULEITEM *item = TAILQ_FIRST(&schedule);
 	int nkilled = 0;
 	int tkilled = 0;
 	int nrunning = 0;
-	long nloops;
+	long nloops, nsecs;
 	struct timespec ts;
 	pid_t pid = 0;
 	const char *const *p;
+	bool progressed = false;
 
 	if (verbose) {
 		if (exec)
@@ -403,12 +415,14 @@ static int run_stop_schedule(const char *exec, const char *const *argv,
 		case SC_SIGNAL:
 			nrunning = 0;
 			nkilled = do_stop(exec, argv, pid, uid, item->value,
-					  quiet, verbose, test);
+			    quiet, verbose, test);
 			if (nkilled == 0) {
 				if (tkilled == 0) {
+					if (progressed)
+						printf("\n");
 					if (! quiet)
 						eerror("%s: no matching "
-						       "processes found", applet);
+						    "processes found", applet);
 				}
 				return tkilled;
 			}
@@ -423,30 +437,49 @@ static int run_stop_schedule(const char *exec, const char *const *argv,
 				break;
 			}
 
-			nloops = (ONE_SECOND / POLL_INTERVAL) * item->value;
 			ts.tv_sec = 0;
 			ts.tv_nsec = POLL_INTERVAL;
 
-			while (nloops) {
-				if ((nrunning = do_stop(exec, argv, pid,
-							uid, 0, true, false, true)) == 0)
-					return 0;
-
-				if (nanosleep(&ts, NULL) == -1) {
-					if (errno == EINTR)
-						eerror("%s: caught an interrupt", applet);
-					else {
-						eerror("%s: nanosleep: %s",
-						       applet, strerror(errno));
+			for (nsecs = 0; nsecs < item->value; nsecs++) {
+				for (nloops = 0;
+				     nloops < ONE_SECOND / POLL_INTERVAL;
+				     nloops++)
+				{
+					if ((nrunning = do_stop(exec, argv,
+						    pid, uid, 0, true, false,
+						    true)) == 0)
 						return 0;
+
+
+					if (nanosleep(&ts, NULL) == -1) {
+						if (progressed) {
+							printf("\n");
+							progressed = false;
+						}
+						if (errno == EINTR)
+							eerror("%s: caught an"
+							    " interrupt", applet);
+						else {
+							eerror("%s: nanosleep: %s",
+							    applet, strerror(errno));
+							return 0;
+						}
 					}
 				}
-					nloops --;
+				if (progress) {
+					printf(".");
+					fflush(stdout);
+					progressed = true;
+				}
 			}
 			break;
-
 		default:
-			eerror("%s: invalid schedule item `%d'", applet, item->type);
+			if (progressed) {
+				printf("\n");
+				progressed = false;
+			}
+			eerror("%s: invalid schedule item `%d'",
+			    applet, item->type);
 			return 0;
 		}
 
@@ -457,17 +490,22 @@ static int run_stop_schedule(const char *exec, const char *const *argv,
 	if (test || (tkilled > 0 && nrunning == 0))
 		return nkilled;
 
+	if (progressed)
+		printf("\n");
 	if (! quiet) {
 		if (nrunning == 1)
-			eerror("%s: %d process refused to stop", applet, nrunning);
+			eerror("%s: %d process refused to stop",
+			    applet, nrunning);
 		else
-			eerror("%s: %d process(es) refused to stop", applet, nrunning);
+			eerror("%s: %d process(es) refused to stop",
+			    applet, nrunning);
 	}
 
 	return -nrunning;
 }
 
-static void handle_signal(int sig)
+static void
+handle_signal(int sig)
 {
 	int status;
 	int serrno = errno;
@@ -492,7 +530,8 @@ static void handle_signal(int sig)
 		for (;;) {
 			if (waitpid(-1, &status, WNOHANG) < 0) {
 				if (errno != ECHILD)
-					eerror("%s: waitpid: %s", applet, strerror(errno));
+					eerror("%s: waitpid: %s",
+					    applet, strerror(errno));
 				break;
 			}
 		}
@@ -533,7 +572,7 @@ expand_home(const char *home, const char *path)
 		ppath++;
 
 	if (!home) {
-		free(opath);
+	free(opath);
 		return xstrdup(path);
 	}
 	if (!ppath) {
@@ -549,7 +588,7 @@ expand_home(const char *home, const char *path)
 }
 
 #include "_usage.h"
-#define getoptstring "KN:R:Sbc:d:e:g:k:mn:op:s:tu:r:x:1:2:" getoptstring_COMMON
+#define getoptstring "KN:PR:Sbc:d:e:g:ik:mn:op:s:tu:r:w:x:1:2:" getoptstring_COMMON
 static const struct option longopts[] = {
 	{ "stop",         0, NULL, 'K'},
 	{ "nicelevel",    1, NULL, 'N'},
@@ -562,6 +601,7 @@ static const struct option longopts[] = {
 	{ "env",          1, NULL, 'e'},
 	{ "umask",        1, NULL, 'k'},
 	{ "group",        1, NULL, 'g'},
+	{ "interpreted",  0, NULL, 'i'},
 	{ "make-pidfile", 0, NULL, 'm'},
 	{ "name",         1, NULL, 'n'},
 	{ "oknodo",       0, NULL, 'o'},
@@ -570,9 +610,11 @@ static const struct option longopts[] = {
 	{ "test",         0, NULL, 't'},
 	{ "user",         1, NULL, 'u'},
 	{ "chroot",       1, NULL, 'r'},
+	{ "wait",         1, NULL, 'w'},
 	{ "exec",         1, NULL, 'x'},
 	{ "stdout",       1, NULL, '1'},
 	{ "stderr",       1, NULL, '2'},
+	{ "progress",     0, NULL, 'P'},
 	longopts_COMMON
 };
 static const char * const longopts_help[] = {
@@ -587,6 +629,7 @@ static const char * const longopts_help[] = {
 	"Set an environment string",
 	"Set the umask for the daemon",
 	"Change the process group",
+	"Match process name by interpreter",
 	"Create a pidfile",
 	"Match process name",
 	"deprecated",
@@ -595,14 +638,17 @@ static const char * const longopts_help[] = {
 	"Test actions, don't do them",
 	"Change the process user",
 	"Chroot to this directory",
+	"Seconds to wait for daemon start",
 	"Binary to start/stop",
 	"Redirect stdout to file",
 	"Redirect stderr to file",
+	"Print dots each second while waiting",
 	longopts_help_COMMON
 };
 #include "_usage.c"
 
-int start_stop_daemon(int argc, char **argv)
+int
+start_stop_daemon(int argc, char **argv)
 {
 	int devnull_fd = -1;
 #ifdef TIOCNOTTY
@@ -627,10 +673,12 @@ int start_stop_daemon(int argc, char **argv)
 	char *name = NULL;
 	char *pidfile = NULL;
 	char *retry = NULL;
-	int sig = 0;
+	int sig = -1;
 	int nicelevel = 0;
 	bool background = false;
 	bool makepidfile = false;
+	bool interpreted = false;
+	bool progress = false;
 	uid_t uid = 0;
 	gid_t gid = 0;
 	char *home = NULL;
@@ -669,7 +717,7 @@ int start_stop_daemon(int argc, char **argv)
 	if ((tmp = getenv("SSD_NICELEVEL")))
 		if (sscanf(tmp, "%d", &nicelevel) != 1)
 			eerror("%s: invalid nice level `%s' (SSD_NICELEVEL)",
-				applet, tmp);
+			    applet, tmp);
 
 	/* Get our user name and initial dir */
 	p = getenv("USER");
@@ -687,15 +735,20 @@ int start_stop_daemon(int argc, char **argv)
 	}
 
 	while ((opt = getopt_long(argc, argv, getoptstring, longopts,
-				  (int *) 0)) != -1)
+		    (int *) 0)) != -1)
 		switch (opt) {
 		case 'K':  /* --stop */
 			stop = true;
 			break;
+
 		case 'N':  /* --nice */
 			if (sscanf(optarg, "%d", &nicelevel) != 1)
 				eerrorx("%s: invalid nice level `%s'",
-					applet, optarg);
+				    applet, optarg);
+			break;
+
+		case 'P':  /* --progress */
+			progress = true;
 			break;
 
 		case 'R':  /* --retry <schedule>|<timeout> */
@@ -712,44 +765,44 @@ int start_stop_daemon(int argc, char **argv)
 
 		case 'u':  /* --user <username>|<uid> */
 		case 'c':  /* --chuid <username>|<uid> */
-			{
-				p = optarg;
-				tmp = strsep(&p, ":");
-				changeuser = xstrdup(tmp);
+		{
+			p = optarg;
+			tmp = strsep(&p, ":");
+			changeuser = xstrdup(tmp);
+			if (sscanf(tmp, "%d", &tid) != 1)
+				pw = getpwnam(tmp);
+			else
+				pw = getpwuid((uid_t)tid);
+
+			if (pw == NULL)
+				eerrorx("%s: user `%s' not found",
+				    applet, tmp);
+			uid = pw->pw_uid;
+			home = pw->pw_dir;
+			unsetenv("HOME");
+			if (pw->pw_dir)
+				setenv("HOME", pw->pw_dir, 1);
+			unsetenv("USER");
+			if (pw->pw_name)
+				setenv("USER", pw->pw_name, 1);
+			if (gid == 0)
+				gid = pw->pw_gid;
+
+			if (p) {
+				tmp = strsep (&p, ":");
 				if (sscanf(tmp, "%d", &tid) != 1)
-					pw = getpwnam(tmp);
+					gr = getgrnam(tmp);
 				else
-					pw = getpwuid((uid_t)tid);
+					gr = getgrgid((gid_t) tid);
 
-				if (pw == NULL)
-					eerrorx("%s: user `%s' not found",
-						applet, tmp);
-				uid = pw->pw_uid;
-				home = pw->pw_dir;
-				unsetenv("HOME");
-				if (pw->pw_dir)
-					setenv("HOME", pw->pw_dir, 1);
-				unsetenv("USER");
-				if (pw->pw_name)
-					setenv("USER", pw->pw_name, 1);
-				if (gid == 0)
-					gid = pw->pw_gid;
-
-				if (p) {
-					tmp = strsep (&p, ":");
-					if (sscanf(tmp, "%d", &tid) != 1)
-						gr = getgrnam(tmp);
-					else
-						gr = getgrgid((gid_t) tid);
-
-					if (gr == NULL)
-						eerrorx("%s: group `%s'"
-							" not found",
-							applet, tmp);
-					gid = gr->gr_gid;
-				}
+				if (gr == NULL)
+					eerrorx("%s: group `%s'"
+					    " not found",
+					    applet, tmp);
+				gid = gr->gr_gid;
 			}
-			break;
+		}
+		break;
 
 		case 'd':  /* --chdir /new/dir */
 			ch_dir = optarg;
@@ -766,14 +819,18 @@ int start_stop_daemon(int argc, char **argv)
 				gr = getgrgid((gid_t)tid);
 			if (gr == NULL)
 				eerrorx("%s: group `%s' not found",
-					applet, optarg);
+				    applet, optarg);
 			gid = gr->gr_gid;
 			break;
 
+		case 'i': /* --interpreted */
+			interpreted = true;
+			break;
+
 		case 'k':
 			if (parse_mode(&numask, optarg))
 				eerrorx("%s: invalid mode `%s'",
-					applet, optarg);
+				    applet, optarg);
 			break;
 
 		case 'm':  /* --make-pidfile */
@@ -807,6 +864,11 @@ int start_stop_daemon(int argc, char **argv)
 		case 'a': /* --startas <name> */
 			startas = optarg;
 			break;
+		case 'w':
+			if (sscanf(optarg, "%d", &start_wait) != 1)
+				eerrorx("%s: `%s' not a number",
+				    applet, optarg);
+			break;
 		case 'x':  /* --exec <executable> */
 			exec = optarg;
 			break;
@@ -820,7 +882,7 @@ int start_stop_daemon(int argc, char **argv)
 			break;
 
 			case_RC_COMMON_GETOPT
-		}
+			    }
 
 	endpwent();
 	argc -= optind;
@@ -854,30 +916,30 @@ int start_stop_daemon(int argc, char **argv)
 	else if (exec)
 		*--argv = exec;
 
-	if (stop || sig != 0) {
-		if (sig == 0)
+	if (stop || sig != -1) {
+		if (sig == -1)
 			sig = SIGTERM;
 		if (!*argv && !pidfile && !name && !uid)
 			eerrorx("%s: --stop needs --exec, --pidfile,"
-				" --name or --user", applet);
+			    " --name or --user", applet);
 		if (background)
 			eerrorx("%s: --background is only relevant with"
-				" --start", applet);
+			    " --start", applet);
 		if (makepidfile)
 			eerrorx("%s: --make-pidfile is only relevant with"
-				" --start", applet);
+			    " --start", applet);
 		if (redirect_stdout || redirect_stderr)
 			eerrorx("%s: --stdout and --stderr are only relevant"
-				" with --start", applet);
+			    " with --start", applet);
 	} else {
 		if (!exec)
 			eerrorx("%s: nothing to start", applet);
 		if (makepidfile && !pidfile)
 			eerrorx("%s: --make-pidfile is only relevant with"
-				" --pidfile", applet);
+			    " --pidfile", applet);
 		if ((redirect_stdout || redirect_stderr) && !background)
 			eerrorx("%s: --stdout and --stderr are only relevant"
-				" with --background", applet);
+			    " with --background", applet);
 	}
 
 	/* Expand ~ */
@@ -894,10 +956,10 @@ int start_stop_daemon(int argc, char **argv)
 			/* Full or relative path */
 			if (ch_root)
 				snprintf(exec_file, sizeof(exec_file),
-					 "%s/%s", ch_root, exec);
+				    "%s/%s", ch_root, exec);
 			else
 				snprintf(exec_file, sizeof(exec_file),
-					 "%s", exec);
+				    "%s", exec);
 		} else {
 			/* Something in $PATH */
 			p = tmp = xstrdup(getenv("PATH"));
@@ -905,11 +967,11 @@ int start_stop_daemon(int argc, char **argv)
 			while ((token = strsep(&p, ":"))) {
 				if (ch_root)
 					snprintf(exec_file, sizeof(exec_file),
-						 "%s/%s/%s",
-						 ch_root, token, exec);
+					    "%s/%s/%s",
+					    ch_root, token, exec);
 				else
 					snprintf(exec_file, sizeof(exec_file),
-						 "%s/%s", token, exec);
+					    "%s/%s", token, exec);
 				if (exists(exec_file))
 					break;
 				*exec_file = '\0';
@@ -919,19 +981,19 @@ int start_stop_daemon(int argc, char **argv)
 	}
 	if (start && !exists(exec_file)) {
 		eerror("%s: %s does not exist", applet,
-			*exec_file ? exec_file : exec);
+		    *exec_file ? exec_file : exec);
 		exit(EXIT_FAILURE);
 	}
 
 	/* If we don't have a pidfile we should check if it's interpreted
 	 * or not. If it we, we need to pass the interpreter through
 	 * to our daemon calls to find it correctly. */
-	if (!pidfile) {
+	if (interpreted && !pidfile) {
 		fp = fopen(exec_file, "r");
 		if (fp) {
-			fgets(line, sizeof(line), fp);
+			p = fgets(line, sizeof(line), fp);
 			fclose(fp);
-			if (line[0] == '#' && line[1] == '!') {
+			if (p != NULL && line[0] == '#' && line[1] == '!') {
 				p = line + 2;
 				/* Strip leading spaces */
 				while (*p == ' ' || *p == '\t')
@@ -958,8 +1020,8 @@ int start_stop_daemon(int argc, char **argv)
 	}
 	margv = nav ? nav : argv;
 
-	if (stop || sig) {
-		if (sig == 0)
+	if (stop || sig != -1) {
+		if (sig == -1)
 			sig = SIGTERM;
 		if (!stop)
 			oknodo = true;
@@ -970,7 +1032,7 @@ int start_stop_daemon(int argc, char **argv)
 		else
 			parse_schedule(NULL, sig);
 		i = run_stop_schedule(exec, (const char *const *)margv,
-				      pidfile, uid, quiet, verbose, test);
+		    pidfile, uid, quiet, verbose, test, progress);
 
 		if (i < 0)
 			/* We failed to stop something */
@@ -986,8 +1048,8 @@ int start_stop_daemon(int argc, char **argv)
 			unlink(pidfile);
 		if (svcname)
 			rc_service_daemon_set(svcname, exec,
-					      (const char *const *)argv,
-					      pidfile, false);
+			    (const char *const *)argv,
+			    pidfile, false);
 		exit(EXIT_SUCCESS);
 	}
 
@@ -997,7 +1059,7 @@ int start_stop_daemon(int argc, char **argv)
 		pid = 0;
 
 	if (do_stop(exec, (const char * const *)margv, pid, uid,
-		    0, true, false, true) > 0)
+		0, true, false, true) > 0)
 		eerrorx("%s: %s is already running", applet, exec);
 
 	if (test) {
@@ -1030,6 +1092,10 @@ int start_stop_daemon(int argc, char **argv)
 		eindent();
 	}
 
+	/* Remove existing pidfile */
+	if (pidfile)
+		unlink(pidfile);
+
 	if (background)
 		signal_setup(SIGCHLD, handle_signal);
 
@@ -1049,30 +1115,35 @@ int start_stop_daemon(int argc, char **argv)
 
 		if (nicelevel) {
 			if (setpriority(PRIO_PROCESS, mypid, nicelevel) == -1)
-				eerrorx("%s: setpritory %d: %s", applet, nicelevel,
-					 strerror(errno));
+				eerrorx("%s: setpritory %d: %s",
+				    applet, nicelevel,
+				    strerror(errno));
 		}
 
 		if (ch_root && chroot(ch_root) < 0)
-			eerrorx("%s: chroot `%s': %s", applet, ch_root, strerror(errno));
+			eerrorx("%s: chroot `%s': %s",
+			    applet, ch_root, strerror(errno));
 
 		if (ch_dir && chdir(ch_dir) < 0)
-			eerrorx("%s: chdir `%s': %s", applet, ch_dir, strerror(errno));
+			eerrorx("%s: chdir `%s': %s",
+			    applet, ch_dir, strerror(errno));
 
 		if (makepidfile && pidfile) {
 			fp = fopen(pidfile, "w");
 			if (! fp)
 				eerrorx("%s: fopen `%s': %s", applet, pidfile,
-					strerror(errno));
+				    strerror(errno));
 			fprintf(fp, "%d\n", mypid);
 			fclose(fp);
 		}
 
 #ifdef HAVE_PAM
 		if (changeuser != NULL)
-			pamr = pam_start("start-stop-daemon", changeuser, &conv, &pamh);
+			pamr = pam_start("start-stop-daemon",
+			    changeuser, &conv, &pamh);
 		else
-			pamr = pam_start("start-stop-daemon", "nobody", &conv, &pamh);
+			pamr = pam_start("start-stop-daemon",
+			    "nobody", &conv, &pamh);
 
 		if (pamr == PAM_SUCCESS)
 			pamr = pam_authenticate(pamh, PAM_SILENT);
@@ -1081,15 +1152,19 @@ int start_stop_daemon(int argc, char **argv)
 		if (pamr == PAM_SUCCESS)
 			pamr = pam_open_session(pamh, PAM_SILENT);
 		if (pamr != PAM_SUCCESS)
-			eerrorx("%s: pam error: %s", applet, pam_strerror(pamh, pamr));
+			eerrorx("%s: pam error: %s",
+			    applet, pam_strerror(pamh, pamr));
 #endif
 
 		if (gid && setgid(gid))
-			eerrorx("%s: unable to set groupid to %d", applet, gid);
+			eerrorx("%s: unable to set groupid to %d",
+			    applet, gid);
 		if (changeuser && initgroups(changeuser, gid))
-			eerrorx("%s: initgroups (%s, %d)", applet, changeuser, gid);
+			eerrorx("%s: initgroups (%s, %d)",
+			    applet, changeuser, gid);
 		if (uid && setuid(uid))
-			eerrorx ("%s: unable to set userid to %d", applet, uid);
+			eerrorx ("%s: unable to set userid to %d",
+			    applet, uid);
 
 		/* Close any fd's to the passwd database */
 		endpwent();
@@ -1112,6 +1187,8 @@ int start_stop_daemon(int argc, char **argv)
 				/* Don't add strings unless they set a var */
 				if (strchr(*pamenv, '='))
 					putenv(xstrdup(*pamenv));
+				else
+					unsetenv(*pamenv);
 				pamenv++;
 			}
 		}
@@ -1119,8 +1196,8 @@ int start_stop_daemon(int argc, char **argv)
 
 		TAILQ_FOREACH(env, env_list, entries) {
 			if ((strncmp(env->value, "RC_", 3) == 0 &&
-			     strncmp(env->value, "RC_SERVICE=", 10) != 0 &&
-			     strncmp(env->value, "RC_SVCNAME=", 10) != 0) ||
+				strncmp(env->value, "RC_SERVICE=", 10) != 0 &&
+				strncmp(env->value, "RC_SVCNAME=", 10) != 0) ||
 			    strncmp(env->value, "SSD_NICELEVEL=", 14) == 0)
 			{
 				p = strchr(env->value, '=');
@@ -1142,8 +1219,8 @@ int start_stop_daemon(int argc, char **argv)
 					while (*p == ':')
 						p++;
 				}
-				if (strcmp(token, RC_LIBDIR "/bin") != 0 &&
-				    strcmp(token, RC_LIBDIR "/sbin") != 0)
+				if (strcmp(token, RC_LIBEXECDIR "/bin") != 0 &&
+				    strcmp(token, RC_LIBEXECDIR "/sbin") != 0)
 				{
 					len = strlen(token);
 					if (np != newpath)
@@ -1161,16 +1238,20 @@ int start_stop_daemon(int argc, char **argv)
 		stdout_fd = devnull_fd;
 		stderr_fd = devnull_fd;
 		if (redirect_stdout) {
-			if ((stdout_fd = open(redirect_stdout, O_WRONLY | O_CREAT | O_APPEND,
-					      S_IRUSR | S_IWUSR)) == -1)
-				eerrorx("%s: unable to open the logfile for stdout `%s': %s",
-					applet, redirect_stdout, strerror(errno));
+			if ((stdout_fd = open(redirect_stdout,
+				    O_WRONLY | O_CREAT | O_APPEND,
+				    S_IRUSR | S_IWUSR)) == -1)
+				eerrorx("%s: unable to open the logfile"
+				    " for stdout `%s': %s",
+				    applet, redirect_stdout, strerror(errno));
 		}
 		if (redirect_stderr) {
-			if ((stderr_fd = open(redirect_stderr, O_WRONLY | O_CREAT | O_APPEND,
-					      S_IRUSR | S_IWUSR)) == -1)
-				eerrorx("%s: unable to open the logfile for stderr `%s': %s",
-					applet, redirect_stderr, strerror(errno));
+			if ((stderr_fd = open(redirect_stderr,
+				    O_WRONLY | O_CREAT | O_APPEND,
+				    S_IRUSR | S_IWUSR)) == -1)
+				eerrorx("%s: unable to open the logfile"
+				    " for stderr `%s': %s",
+				    applet, redirect_stderr, strerror(errno));
 		}
 
 		/* We don't redirect stdin as some daemons may need it */
@@ -1189,7 +1270,7 @@ int start_stop_daemon(int argc, char **argv)
 			pam_close_session(pamh, PAM_SILENT);
 #endif
 		eerrorx("%s: failed to exec `%s': %s",
-			applet, exec,strerror(errno));
+		    applet, exec,strerror(errno));
 	}
 
 	/* Parent process */
@@ -1202,13 +1283,15 @@ int start_stop_daemon(int argc, char **argv)
 		do {
 			pid = waitpid(spid, &i, 0);
 			if (pid < 1) {
-				eerror("waitpid %d: %s", spid, strerror(errno));
+				eerror("waitpid %d: %s",
+				    spid, strerror(errno));
 				return -1;
 			}
 		} while (!WIFEXITED(i) && !WIFSIGNALED(i));
 		if (!WIFEXITED(i) || WEXITSTATUS(i) != 0) {
 			if (!quiet)
-				eerrorx("%s: failed to start `%s'", applet, exec);
+				eerrorx("%s: failed to start `%s'",
+				    applet, exec);
 			exit(EXIT_FAILURE);
 		}
 		pid = spid;
@@ -1216,83 +1299,55 @@ int start_stop_daemon(int argc, char **argv)
 
 	/* Wait a little bit and check that process is still running
 	   We do this as some badly written daemons fork and then barf */
-	if ((p = getenv("SSD_STARTWAIT")) ||
-	    (p = rc_conf_value("rc_start_wait")))
+	if (start_wait == 0 &&
+	    ((p = getenv("SSD_STARTWAIT")) ||
+		(p = rc_conf_value("rc_start_wait"))))
 	{
-		if (sscanf(p, "%u", &start_wait) == 1)
-			start_wait *= 1000000;
-		else
+		if (sscanf(p, "%u", &start_wait) != 1)
 			start_wait = 0;
 	}
 
 	if (start_wait > 0) {
 		struct timespec ts;
-		int nloops = start_wait / POLL_INTERVAL;
-		int nloopsp = WAIT_PIDFILE / POLL_INTERVAL;
 		bool alive = false;
-
-		ts.tv_sec = 0;
-		ts.tv_nsec = POLL_INTERVAL;
-
-		while (nloops) {
-			if (nanosleep(&ts, NULL) == -1) {
-				if (errno == EINTR)
-					eerror("%s: caught an interrupt", applet);
-				else {
-					eerror("%s: nanosleep: %s", applet, strerror(errno));
-					return 0;
-				}
+		
+		ts.tv_sec = start_wait;
+		ts.tv_nsec = 0;
+		if (nanosleep(&ts, NULL) == -1) {
+			if (errno == EINTR)
+				eerror("%s: caught an interrupt", applet);
+			else {
+				eerror("%s: nanosleep: %s",
+				    applet, strerror(errno));
+				return 0;
 			}
-
-			/* We wait for a specific amount of time for a pidfile
-			 * to be created.
-			 * Once everything is in place we then wait some more
-			 * to ensure that the daemon really is running and won't
-			 * abort due to a config error. */
-			if (!background && pidfile && nloopsp)
-				nloopsp--;
-			else
-				nloops--;
-
-			/* This is knarly.
-			 * If we backgrounded then we know the exact pid.
-			 * Otherwise if we have a pidfile then it *may* know
-			 * the exact pid.
-			 * Failing that, we'll have to query processes.
-			 * We sleep first as some programs like ntp like to
-			 * fork, and write their pidfile a LONG time later. */
-			if (background) {
-				if (kill (pid, 0) == 0)
-					alive = true;
-			} else {
-				if (pidfile) {
-					/* The pidfile may not have been
-					 * written yet - give it some time */
-					if ((pid = get_pid(pidfile, true)) == -1) {
-						if (! nloopsp)
-							eerrorx("%s: did not "
-								"create a valid"
-								" pid in `%s'",
-								applet, pidfile);
-						alive = true;
-						pid = 0;
-					} else
-						nloopsp = 0;
-				} else
-					pid = 0;
-				if (do_stop(exec, (const char *const *)margv,
-					    pid, uid, 0, true, false, true) > 0)
-					alive = true;
-			}
-
-			if (!alive)
-				eerrorx("%s: %s died", applet, exec);
 		}
+		if (background) {
+			if (kill(pid, 0) == 0)
+				alive = true;
+		} else {
+			if (pidfile) {
+				pid = get_pid(pidfile, true);
+				if (pid == -1) {
+					eerrorx("%s: did not "
+					    "create a valid"
+					    " pid in `%s'",
+					    applet, pidfile);
+				}
+			} else
+				pid = 0;
+			if (do_stop(exec, (const char *const *)margv,
+				pid, uid, 0, true, false, true) > 0)
+				alive = true;
+		}
+
+		if (!alive)
+			eerrorx("%s: %s died", applet, exec);
 	}
 
 	if (svcname)
-		rc_service_daemon_set(svcname, exec, (const char *const *)margv,
-				      pidfile, true);
+		rc_service_daemon_set(svcname, exec,
+		    (const char *const *)margv, pidfile, true);
 
 	exit(EXIT_SUCCESS);
 	/* NOTREACHED */
